# Env var names
ANSIBLE_SSH_ARGS = "ANSIBLE_SSH_ARGS"
ANSIBLE_DEBUG = "ANSIBLE_DEBUG"
ANSIBLE_SSH_EXECUTABLE = "ANSIBLE_SSH_EXECUTABLE"
ANSIBLE_SSH_RETRIES = "ANSIBLE_SSH_RETRIES"
ANSIBLE_PIPELINING = "ANSIBLE_PIPELINING"
SYM_USE_CONTROL_MASTER = "SYM_USE_CONTROL_MASTER"
SYM_DEBUG = "SYM_DEBUG"
SYM_LOG_DIR = "SYM_LOG_DIR"
OBJC_DISABLE_INITIALIZE_FORK_SAFETY = "OBJC_DISABLE_INITIALIZE_FORK_SAFETY"
ANSIBLE_CONNECTION_PLUGINS = "ANSIBLE_CONNECTION_PLUGINS"


# Env var values
DEFAULT_ANSIBLE_SSH_RETRIES = str(2)
ANSIBLE_SSH_ARGS_NO_CONTROL_MASTER = "-C -o ControlMaster=no"
ANSIBLE_SSH_ARGS_DEBUG = "-o Compression=no -o ControlMaster=no"
DEFAULT_ANSIBLE_SSH_ARGS = "-C -o ControlMaster=auto -o ControlPersist=15m"
ENABLED = str(1)
YES = "YES"
